"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Vehicle = void 0;
const axios_1 = __importDefault(require("axios"));
const vehicle_1 = require("./types/vehicle");
const events_1 = require("events");
const defaultHeaders = {
    'Content-Type': 'application/json',
    'User-Agent': 'FordPass/5 CFNetwork/1333.0.4 Darwin/21.5.0',
};
const defaultAppId = '71A3AD0A-CF46-4CCF-B473-FC7FE5BC4592';
const fordAPIUrl = 'https://usapi.cv.ford.com/';
const handleError = function (name, status, log) {
    log.error(`${name} failed with status: ${status}`);
};
class Vehicle extends events_1.EventEmitter {
    constructor(name, vin, config, log) {
        var _a, _b, _c;
        super();
        this.updating = false;
        this.config = config;
        this.log = log;
        this.name = name;
        this.vin = vin;
        this.autoRefresh = ((_a = config.options) === null || _a === void 0 ? void 0 : _a.autoRefresh) || false;
        this.refreshRate = ((_b = config.options) === null || _b === void 0 ? void 0 : _b.refreshRate) || 180;
        this.applicationId = ((_c = config.options) === null || _c === void 0 ? void 0 : _c.region) || defaultAppId;
    }
    status() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.config.access_token) {
                return;
            }
            const url = fordAPIUrl + `/api/vehicles/v5/${this.vin}/status`;
            const options = {
                url: url,
                headers: defaultHeaders,
                params: {
                    lrdt: '01-01-1970 00:00:00',
                },
            };
            if (options.headers) {
                options.headers['Application-Id'] = this.applicationId;
                options.headers['auth-token'] = this.config.access_token;
            }
            if (!this.updating) {
                this.updating = true;
                try {
                    const result = yield (0, axios_1.default)(options);
                    if (result.status === 200 && result.data.status === 200) {
                        this.info = result.data.vehiclestatus;
                        return this.info;
                    }
                    else if (result.data.status === 401) {
                        this.log.error(`You do not have authorization to access ${this.name}.`);
                    }
                    else {
                        handleError('Status', result.data.status, this.log);
                    }
                }
                catch (error) {
                    if (error.code !== 'ETIMEDOUT') {
                        this.log.error(`Status failed with error: ${error.code || error.response.status}`);
                    }
                }
                finally {
                    this.updating = false;
                    this.emit('updated');
                }
            }
            else {
                yield (0, events_1.once)(this, 'updated');
                return this.info;
            }
        });
    }
    issueCommand(command) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.config.access_token) {
                return '';
            }
            let method = 'GET';
            let endpoint = '';
            switch (command) {
                case vehicle_1.Command.START: {
                    method = 'PUT';
                    endpoint = `api/vehicles/v5/${this.vin}/engine/start`;
                    break;
                }
                case vehicle_1.Command.STOP: {
                    method = 'DELETE';
                    endpoint = `api/vehicles/v5/${this.vin}/engine/start`;
                    break;
                }
                case vehicle_1.Command.LOCK: {
                    method = 'PUT';
                    endpoint = `api/vehicles/v5/${this.vin}/doors/lock`;
                    break;
                }
                case vehicle_1.Command.UNLOCK: {
                    method = 'DELETE';
                    endpoint = `api/vehicles/v5/${this.vin}/doors/lock`;
                    break;
                }
                case vehicle_1.Command.REFRESH: {
                    method = 'PUT';
                    endpoint = `api/vehicles/v5/${this.vin}/status`;
                    break;
                }
                default: {
                    this.log.error('invalid command');
                    break;
                }
            }
            if (endpoint) {
                const url = fordAPIUrl + endpoint;
                const options = {
                    method: method,
                    url: url,
                    headers: defaultHeaders,
                };
                if (options.headers) {
                    options.headers['Application-Id'] = this.applicationId;
                    options.headers['auth-token'] = this.config.access_token;
                }
                try {
                    const result = yield (0, axios_1.default)(options);
                    if (result.status !== 200) {
                        handleError('IssueCommand', result.status, this.log);
                        return '';
                    }
                    return result.data.commandId;
                }
                catch (error) {
                    this.log.error(`Command failed with error: ${error.code || error.response.status}`);
                }
            }
            return '';
        });
    }
    commandStatus(command, commandId) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.config.access_token) {
                return;
            }
            let endpoint = '';
            if (command === vehicle_1.Command.START || command === vehicle_1.Command.STOP) {
                endpoint = `api/vehicles/v5/${this.vin}/engine/start/${commandId}`;
            }
            else if (command === vehicle_1.Command.LOCK || command === vehicle_1.Command.UNLOCK) {
                endpoint = `api/vehicles/v5/${this.vin}/doors/lock/${commandId}`;
            }
            else if (command === vehicle_1.Command.REFRESH) {
                endpoint = `api/vehicles/v5/${this.vin}/status/${commandId}`;
            }
            else {
                this.log.error('invalid command');
            }
            const url = fordAPIUrl + endpoint;
            const options = {
                baseURL: fordAPIUrl,
                url: url,
                headers: defaultHeaders,
            };
            if (options.headers) {
                options.headers['Application-Id'] = this.applicationId;
                options.headers['auth-token'] = this.config.access_token;
            }
            try {
                const result = yield (0, axios_1.default)(options);
                if (result.status === 200) {
                    return result.data;
                }
                else {
                    handleError('CommandStatus', result.status, this.log);
                }
            }
            catch (error) {
                this.log.error(`CommandStatus failed with error: ${error.code || error.response.status}`);
            }
            return;
        });
    }
}
exports.Vehicle = Vehicle;
//# sourceMappingURL=fordpass.js.map